# Instance Matrix
The table below defines the instances of test hardware available on the `ubxlib` test farm and how each one is currently configured.  It is parsed and used by the functions in `u_data.py` to know what to do so please always ensure that it is a properly formatted Markdown format table.

The first digit of the instance ID represents either a physical thing: a board connected via a COM port/debugger, or a distinct process (e.g. run Lint).  The remaining digits represent variants, e.g. using different SDKs on a board.

The platform field must match a unique platform name from under one of the `platform/<vendor>/<chipset>` directories (e.g `esp32`), the SDK must match an `sdk` of that platform, each API must be the name of one in the directory structure (e.g. `port` for the porting API), module names must be used consistently as must the second\[/third\] digit(s) of the instance ID (e.g. for NRF52 variant 0 is always the GCC build variant, variant 1 is always the SES build variant); case is ignored.  If a module name begins with `SARA` it will be prefixed by the test automation code with `U_CELL_MODULE_TYPE_` and then assigned to a compilation flag `U_CFG_TEST_CELL_MODULE_TYPE` for use during testing.

|  ID   | Description                                | Platform  |     SDK    | Module(s) (separate with spaces) |  APIs available (separate with spaces)      | #define overrides (separate with spaces) |
| :---: | ------------------------------------------ | :-------: | :--------: | :------------------------------: | ------------------------------------------- | ---------------------------------------- |
| 0     | Run Lint                                   |           |            |                                  |                                             |                                          |
| 1     | Run Doxygen                                |           |            |                                  |                                             |                                          |
| 2     | Run AStyle style checker (for advice only) |           |            |                                  |                                             |                                          |
| 3     | Run Pylint                                 |           |            |                                  |                                             |                                          |
| 4     | Run Lint with logging off                  |           |            |                                  |                                             | U_CFG_ENABLE_LOGGING=0                   |
| 5     | Reserved                                   |           |            |                                  |                                             |                                          |
| 6     | Reserved                                   |           |            |                                  |                                             |                                          |
| 7     | Reserved                                   |           |            |                                  |                                             |                                          |
| 8     | Reserved                                   |           |            |                                  |                                             |                                          |
| 9     | Reserved                                   |           |            |                                  |                                             |                                          |
| 10    | WHRE board (NINA-W1), Cat M1               |   ESP32   |   ESP-IDF  | SARA_R412M_03B                   | port cell sock network                      | U_CFG_TEST_PIN_A=-1 U_CFG_TEST_PIN_B=-1 U_CFG_TEST_PIN_C=-1 U_CFG_TEST_UART_A=-1 |
| 11    | ESP32-DevKitC + EVK                        |   ESP32   |   ESP-IDF  |                                  | port at_client                              | U_CFG_TEST_UART_B=1 U_CFG_TEST_PIN_UART_A_CTS=-1 U_CFG_TEST_PIN_UART_A_RTS=-1 U_CFG_TEST_PIN_UART_A_RXD=26 U_CFG_TEST_PIN_UART_B_TXD=27 U_CFG_TEST_PIN_UART_B_RXD=14 |
| 12    | ESP32-DevKitC + EVK, Cat M1                |   ESP32   |   ESP-IDF  | SARA_R5                          | port cell sock network short_range security | U_CFG_APP_PIN_CELL_TXD=21 U_CFG_APP_PIN_CELL_RXD=19 U_CFG_APP_PIN_CELL_VINT=-1 U_CFG_APP_PIN_CELL_ENABLE_POWER=-1 |
| 13.0  | Nordic DK board (NRF52840) + EVK, Cat M1   |   NRF52   |     GCC    | SARA_R5                          | port at_client cell sock network security   | U_CFG_TEST_UART_B=0 U_CFG_TEST_PIN_UART_A_CTS=-1 U_CFG_TEST_PIN_UART_A_RTS=-1 U_CFG_TEST_PIN_UART_B_TXD=44 U_CFG_TEST_PIN_UART_B_RXD=43 U_CFG_TEST_PIN_UART_A_RXD=45 |
| 13.1  | Nordic DK board (NRF52840) + EVK           |   NRF52   |     SES    |                                  | port at_client                              | U_CFG_TEST_UART_B=0 U_CFG_TEST_PIN_UART_A_CTS=-1 U_CFG_TEST_PIN_UART_A_RTS=-1 U_CFG_TEST_PIN_UART_B_TXD=44 U_CFG_TEST_PIN_UART_B_RXD=43 U_CFG_TEST_PIN_UART_A_RXD=45 |
| x     | C030-R412M board (STM32F437), GPRS         |  STM32F4  |    Cube    | SARA_R412M_02B                   | port cell sock network                      | U_CELL_NET_TEST_RAT=U_CELL_NET_RAT_GSM_GPRS_EGPRS |
| 15    | STM32F4 Discovery (STM32F407) + EVK, Cat M1|  STM32F4  |    Cube    | SARA_R412M_02B                   | port cell sock network security             | HSE_VALUE=((uint32_t)8000000U) U_CFG_TEST_UART_A=-1 U_CFG_APP_PIN_C030_ENABLE_3V3=-1 U_CFG_APP_PIN_CELL_RESET=-1 U_CFG_APP_CELL_UART=3 U_CFG_APP_PIN_CELL_TXD=0x38 U_CFG_APP_PIN_CELL_RXD=0x39 U_CFG_APP_PIN_CELL_RTS=-1 U_CFG_APP_PIN_CELL_CTS=-1 |
| 16.0  | Nordic DK board (NRF52840) + EVK           |   NRF52   |     GCC    |                                  | port                                        |                                          |
| 16.1  | Nordic DK board (NRF52840) + EVK, Cat M1   |   NRF52   |     SES    |                                  | port                                        |                                          |
| 17.0  | NINA-B3 (on a C208 board)                  |   NRF52   |     GCC    |                                  | port                                        | U_CFG_TEST_PIN_A=-1 U_CFG_TEST_PIN_B=-1 U_CFG_TEST_PIN_C=-1 U_CFG_TEST_UART_A=-1 |
| 17.1  | NINA-B3 (on a C208 board)                  |   NRF52   |     SES    |                                  | port short_range                            | U_CFG_TEST_PIN_A=-1 U_CFG_TEST_PIN_B=-1 U_CFG_TEST_PIN_C=-1 U_CFG_TEST_UART_A=-1 |
| 18    | C030-U201 board (STM32F437), 3G            |  STM32F4  |    Cube    | SARA_U201                        | port at_client cell sock network            | U_CFG_APP_CELL_UART=2 U_CFG_APP_PIN_CELL_TXD=0x35 U_CFG_APP_PIN_CELL_RXD=0x36 U_CFG_APP_PIN_CELL_RTS=0x34 U_CFG_APP_PIN_CELL_CTS=0x33 U_CFG_TEST_UART_B=1 U_CFG_TEST_PIN_UART_B_TXD=0x16 U_CFG_TEST_PIN_UART_B_RXD=0x17 U_CFG_TEST_PIN_UART_A_RTS=-1 U_CFG_TEST_PIN_UART_A_CTS=-1 |
| 19    | Nordic NRF53 board                         |   NRF53   | nRFConnect |                                  | port short_range                            |                                          |
| 20    | Nordic NRF53 board + EVK, Cat M1           |   NRF53   | nRFConnect | SARA_R5                          | port cell sock network security             | U_CFG_TEST_PIN_A=-1 U_CFG_TEST_PIN_B=-1 U_CFG_TEST_PIN_C=-1 U_CFG_TEST_UART_A=-1 U_CFG_APP_CELL_UART=1 |
| 21    | C030-R5 board (STM32F437), Cat M1          |  STM32F4  |    Cube    | SARA_R5                          | port cell sock network short_range security | U_CFG_TEST_PIN_A=-1 U_CFG_TEST_PIN_B=-1 U_CFG_TEST_PIN_C=-1 U_CFG_TEST_UART_A=-1 |

Notes:
- Don't change existing instance IDs without checking the rest of the automation system for scripts which "know" about their numbers (e.g. the `esp32` ones do).
- The "APIs available" column indicates which APIs are available to *test* on a given instance: other APIs may be available on that instance but the ones listed in that column are the ones that are intended to be tested on that instance (since in some cases specific board capabilites/wiring are required).
- The #defines listed are *overrides* on the default values that are defined in the code or additional to those defined in the code; they are not a complete list.
- The instances on which `at_client` is available to test use the same wiring as the other boards but allocate UARTs to pins differently: two UARTs are used for back-to-back testing, no flow control, and the CTS/RTS pins that are back-to-back connected for `port` UART testing are instead allocated to the second UART; on these instances the `port` UART test will be skipped.
- The cellular tests take quite a long time to run and so where there are two SDKs on a single platform usually only one of them will run the cellular tests to keep the execution time down.

# Maintenance
- When adding a new API make sure that it is present in at least one of the entries above, otherwise it will NOT be detected and tested at all on a Pull Request.